unit MainFrm;

interface

uses
  Windows,
  Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, MRVCamMultiView, MRVCamView, MRVCamera, ComCtrls,
  ExtCtrls, MRVType, MRVMediaSource, MRVBitmap;

type
  TfrmMain = class(TForm)
    RVCamera1: TRVCamera;
    RVCamera2: TRVCamera;
    RVCamera3: TRVCamera;
    RVCamera4: TRVCamera;
    lstCameras: TListBox;
    RVCamMultiView1: TRVCamMultiView;
    StatusBar1: TStatusBar;
    Label1: TLabel;
    Panel1: TPanel;
    tbBright: TTrackBar;
    procedure RVCamera1SearchComplete(Sender: TObject; Status: Integer);
    procedure FormCreate(Sender: TObject);
    procedure lstCamerasClick(Sender: TObject);
    procedure RVCamMultiView1SelectViewer(Sender: TObject; Viewer: TRVCamView;
      ViewerIndex: Integer);
    procedure RVCamMultiView1ViewerPaint(Sender: TObject; Viewer: TRVCamView; ViewerIndex: Integer; VideoFrame: TBitmap; ACanvas: TCanvas; var CanDrawFrame: Boolean);
  private
    FVideoFrame: TRVMBitmap;
  public
    procedure SetParams(Cam: TRVCamera);
    function GetCameraIndex(Cam: TRVCamera): Integer;
    function GetSelectedCamera: TRVCamera;
    procedure StatusChange(Camera: TRVCamera; const Status: String);
    procedure EnableControls(Enabled: Boolean);
  end;

var
  frmMain: TfrmMain;

implementation

uses CamList, ImgFilter;

{$R *.dfm}

{------------------------------------------------------------------------------}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FVideoFrame:=TRVMBitmap.Create;
  SetParams(RVCamera1);
  SetParams(RVCamera2);
  SetParams(RVCamera3);
  SetParams(RVCamera4);
  CamListLoader.Load(lstCameras.Items);
  CamListLoader.OnStatusChange := StatusChange;
  CamListLoader.OnEnableControls := EnableControls;

  lstCameras.ItemIndex := 0;
  lstCameras.OnClick(nil);
end;

{------------------------------------------------------------------------------}
procedure TfrmMain.SetParams(Cam: TRVCamera);
begin
  Cam.OnEndVideoStream := CamListLoader.DoOnEndVideoStream;
  Cam.OnVideoStart := CamListLoader.DoOnVideoStart;
  Cam.OnStartVideoStream := CamListLoader.DoOnStartVideoStream;
end;

{------------------------------------------------------------------------------}
function TfrmMain.GetSelectedCamera: TRVCamera;
begin
  Result := RVCamMultiView1.Viewers[RVCamMultiView1.ViewerIndex]
    .VideoSource as TRVCamera;
end;

{------------------------------------------------------------------------------}
function TfrmMain.GetCameraIndex(Cam: TRVCamera): Integer;
begin
  if Cam = RVCamera1 then
    Result := 0
  else if Cam = RVCamera2 then
    Result := 1
  else if Cam = RVCamera3 then
    Result := 2
  else if Cam = RVCamera4 then
    Result := 3
  else
    Result := -1;
end;

{------------------------------------------------------------------------------}
// This procedure is called when the user chooses a camera from the list
procedure TfrmMain.lstCamerasClick(Sender: TObject);
begin
  if RVCamMultiView1.ViewerIndex < 0 then
    exit;
  CamListLoader.Select(lstCameras.ItemIndex, GetSelectedCamera);
end;

{------------------------------------------------------------------------------}
// This procedure is called when the camera searching is complete
procedure TfrmMain.RVCamera1SearchComplete(Sender: TObject; Status: Integer);
begin
  CamListLoader.SearchComplete(Sender as TRVCamera, Status,
    Sender = GetSelectedCamera);
end;

{------------------------------------------------------------------------------}
// This procedure is called by CamListLoader to display a status message
// We display messages from all cameras in the same status bar
procedure TfrmMain.StatusChange(Camera: TRVCamera; const Status: String);
begin
  if not (csDestroying in ComponentState) then
    StatusBar1.Panels[GetCameraIndex(Camera)].Text := Status;
end;

{------------------------------------------------------------------------------}
// This procedure is used by CamListLoader to disable controls while performing
// long operations, and to enable them back
procedure TfrmMain.EnableControls(Enabled: Boolean);
begin
  if not (csDestroying in ComponentState) then
    lstCameras.Enabled := Enabled;
end;

procedure TfrmMain.RVCamMultiView1SelectViewer(Sender: TObject;
  Viewer: TRVCamView; ViewerIndex: Integer);
begin
  EnableControls(not GetSelectedCamera.Searching);
end;

procedure TfrmMain.RVCamMultiView1ViewerPaint(Sender: TObject; Viewer: TRVCamView; ViewerIndex: Integer; VideoFrame: TBitmap; ACanvas: TCanvas; var CanDrawFrame: Boolean);
begin
  if (ViewerIndex<>4) or (VideoFrame.Width=0) then
    Exit;
  FVideoFrame.Assign(VideoFrame);
  CanDrawFrame:=False;
  GBlur(FVideoFrame,3);
  ACanvas.StretchDraw(Viewer.FrameRect,FVideoFrame.GetBitmap);
end;

end.
